#' Plotting cellphonedb results
#'
#' @param cell_type1 cell type 1
#' @param cell_type2 cell type 2
#' @param scdata single-cell data. can be seurat/summarizedexperiment object
#' @param idents vector holding the idents for each cell or column name of scdata's metadata. MUST match cpdb's columns
#' @param means object holding means.txt from cpdb output
#' @param pvals object holding pvals.txt from cpdb output
#' @param deconvoluted object holding deconvoluted.txt from cpdb output
#' @param p.adjust.method correction method. p.adjust.methods of one of these options: c('holm', 'hochberg', 'hommel', 'bonferroni', 'BH', 'BY', 'fdr', 'none')
#' @param keep_significant_only logical. Default is FALSE. Switch to TRUE if you only want to plot the significant hits from cpdb.
#' @param split.by column name in the metadata/coldata table to split the spots by. Can only take columns with binary options. If specified, name to split by MUST be specified in the meta file provided to cpdb prior to analysis.
#' @param scale logical. scale the expression to mean +/- SD. NULL defaults to TRUE.
#' @param standard_scale logical. scale the expression to range from 0 to 1. Default is TRUE
#' @param separator default = NULL. separator to use to split between celltypes. Unless otherwise specified, the separator will be `>@<`. Make sure the idents and split.by doesn't overlap with this.
#' @param gene_symbol_mapping default = NULL.column name for rowData in sce holding the actual gene symbols if row names aren't gene symbols
#' @param frac default = 0.1. Minimum fraction of celtypes expressing a gene in order to keep the interaction. Gene must be expressesd >= `frac` in either of the pair of celltypes in order to keep.
#' @param remove_self default = TRUE. Remove self-self arcs.
#' @param desiredInteractions default = NULL. Specific list of celltype comparisons e.g. list(c('CD4_Tcm', 'cDC1'), c('CD4_Tcm', 'cDC2')). Also accepts a dataframe where first column is celltype 1 and 2nd column is celltype 2.
#' @param interaction_grouping default = NULL. dataframe specifying groupings of cellphonedb interactions. First column must be cellphonedb's interacting_pair column. second column is whatever grouping you want.
#' @param edge_group_colors default = NULL. vector for colour mapping for edge groups. only used if split.by is specified.
#' @param node_group_colors default = NULL. vector for colour mapping for node labels.
#' @param degs_analysis if is cellphonedb degs_analysis mode. 
#' @param return_df whether to just return this as a data.frame rather than plotting iot
#' @param ... passes arguments plot_cpdb
#' @return Plotting cellphonedb results as a weird chord diagram
#' @examples
#' \donttest{
#' }
#' @include utils.R
#' @import ggplot2
#' @import ggraph
#' @import ggrepel
#' @export

plot_cpdb2 <- function(cell_type1, cell_type2, scdata, idents, means, pvals, deconvoluted,
    p.adjust.method = NULL, keep_significant_only = TRUE, split.by = NULL, standard_scale = TRUE,
    separator = NULL, gene_symbol_mapping = NULL, frac = 0.1, remove_self = TRUE,
    desiredInteractions = NULL, interaction_grouping = NULL, edge_group_colors = NULL,
    node_group_colors = NULL, degs_analysis = FALSE, return_df = FALSE, ...) {
    if (class(scdata) == "Seurat") {
        stop("Sorry not supported. Please use a SingleCellExperiment object.")
    }
    if (length(separator) > 0) {
        sep = separator
    } else {
        sep = ">@<"
    }
    lr_interactions = plot_cpdb(cell_type1 = cell_type1, cell_type2 = cell_type2, scdata = scdata,
        idents = idents, split.by = split.by, means = means, pvals = pvals, keep_significant_only = keep_significant_only,
        standard_scale = standard_scale, return_table = TRUE, degs_analysis = degs_analysis, ...)
    requireNamespace("SummarizedExperiment")
    requireNamespace("SingleCellExperiment")
    if (is.null(split.by)) {
        if (any(lr_interactions[, 3] > 0)) {
            if (any(is.na(lr_interactions[, 3]))) {
                lr_interactions <- lr_interactions[lr_interactions[, 3] > 0 & !is.na(lr_interactions[,
                  3]), ]
            } else {
                lr_interactions <- lr_interactions[lr_interactions[, 3] > 0, ]
            }
        }
    }
    subset_clusters <- unique(unlist(lapply(as.character(lr_interactions$group),
        strsplit, sep)))
    sce_subset <- scdata[, SummarizedExperiment::colData(scdata)[, idents] %in% subset_clusters]
    interactions <- means[, c("interacting_pair", "gene_a", "gene_b", "partner_a",
        "partner_b", "receptor_a", "receptor_b")]
    interactions$converted <- gsub("-", " ", interactions$interacting_pair)
    interactions$converted <- gsub("_", "-", interactions$converted)
    interactions_subset <- interactions[interactions$converted %in% lr_interactions$Var1,
        ]
    tm0 <- do.call(c, lapply(as.list(interactions_subset$interacting_pair), strsplit,
        "_"))
    if (any(lapply(tm0, length) > 2)) {
        complex_id <- which(lapply(tm0, length) > 2)
        interactions_subset_ = interactions_subset[complex_id, ]
        simple_1 = interactions_subset_$interacting_pair[grep("complex:", interactions_subset_$partner_b)]
        partner_1 = gsub("complex:", "", interactions_subset_$partner_b[grep("complex:",
            interactions_subset_$partner_b)])
        partner_2 = gsub("complex:", "", interactions_subset_$partner_a[grep("complex:",
            interactions_subset_$partner_a)])
        simple_2 = interactions_subset_$interacting_pair[grep("complex:", interactions_subset_$partner_a)]
        for (i in seq_along(simple_1)) {
            simple_1[i] <- gsub(paste0(partner_1[i], "_|_", partner_1[i]), "", simple_1[i])
        }
        for (i in seq_along(simple_2)) {
            simple_2[i] <- gsub(paste0(partner_2[i], "_|_", partner_2[i]), "", simple_2[i])
        }
        tmpdf <- rbind(cbind(simple_1, partner_1), cbind(partner_2, simple_2))
        tmplist <- split(tmpdf, seq(nrow(tmpdf)))
        for (i in seq_along(complex_id)) {
            tm0[[complex_id[i]]] <- tmplist[[i]]
        }
        tm0 = data.frame(t(matrix(unlist(tm0), 2, length(unlist(tm0))/2)))
        colnames(tm0) <- c("id_a", "id_b")
        interactions_subset <- cbind(interactions_subset, tm0)
        dictionary <- interactions_subset[, c("gene_a", "gene_b", "partner_a", "partner_b",
            "id_a", "id_b", "receptor_a", "receptor_b")]
    } else {
        tm0 = data.frame(t(matrix(unlist(tm0), 2, length(unlist(tm0))/2)))
        colnames(tm0) <- c("id_a", "id_b")
        interactions_subset <- cbind(interactions_subset, tm0)
        dictionary <- interactions_subset[, c("gene_a", "gene_b", "partner_a", "partner_b",
            "id_a", "id_b", "receptor_a", "receptor_b")]
    }
    if (!is.null(interaction_grouping)) {
        if ((class(interaction_grouping) == "data.frame")) {
            interactions_subset$group = interaction_grouping[, 2][match(interactions_subset$interacting_pair,
                interaction_grouping[, 1])]
        }
    }
    # extract all the possible genes.
    geneid = unique(c(interactions_subset$id_a, interactions_subset$id_b))
    # rmg = which(geneid == '') if (length(rmg) > 0){ geneid = geneid[-which(geneid
    # == '')] }
    if (all(!geneid %in% row.names(sce_subset))) {
        geneid = unique(c(interactions_subset$gene_a, interactions_subset$gene_b))
    }
    sce_subset_tmp <- sce_subset[row.names(sce_subset) %in% geneid, ]

    if (dim(sce_subset_tmp)[1] == 0) {
        stop("Gene ids not found. Are you sure your single-cell object contains human gene symbols?")
    }
    # split to list and calculate celltype mean for each treatment group
    meta <- as.data.frame(SummarizedExperiment::colData(sce_subset_tmp))
    if (!is.null(split.by)) {
        sce_list <- list()
        sce_list_alt <- list()
        for (x in unique(meta[, split.by])) {
            sce_list[[x]] <- list()
            sce_list_alt[[x]] <- list()
        }
        for (n in names(sce_list)) {
            for (x in unique(meta[, idents])) {
                sce_list[[n]][[x]] <- sce_subset_tmp[, meta[, idents] == x & meta[,
                  split.by] == n]
                sce_list_alt[[n]][[x]] <- sce_subset[, meta[, idents] == x & meta[,
                  split.by] == n]
            }
        }
        sce_list2 <- lapply(sce_list, function(y) {
            z <- lapply(y, .cellTypeMeans)
            return(z)
        })
        sce_list3 <- lapply(sce_list, function(y) {
            z <- lapply(y, .cellTypeFraction)
            return(z)
        })
        sce_list2 <- lapply(sce_list2, function(x) do.call(cbind, x))
        sce_list3 <- lapply(sce_list3, function(x) do.call(cbind, x))
        for (n in names(sce_list2)) {
            colnames(sce_list2[[n]]) <- paste0(paste0(n, "_"), colnames(sce_list2[[n]]))
            colnames(sce_list3[[n]]) <- paste0(paste0(n, "_"), colnames(sce_list3[[n]]))
        }
        sce_list2 <- do.call(cbind, sce_list2)
        sce_list3 <- do.call(cbind, sce_list3)
    } else {
        sce_list <- list()
        sce_list_alt <- list()
        for (x in unique(meta[, idents])) {
            sce_list[[x]] <- sce_subset_tmp[, meta[, idents] == x]
            sce_list_alt[[x]] <- sce_subset[, meta[, idents] == x]
        }
        sce_list2 <- lapply(sce_list, .cellTypeMeans)
        sce_list3 <- lapply(sce_list, .cellTypeFraction)
        sce_list2 <- do.call(cbind, sce_list2)
        sce_list3 <- do.call(cbind, sce_list3)
    }
    keep_a <- which(dictionary$gene_a != "")
    keep_b <- which(dictionary$gene_b != "")
    id_a_dict <- dictionary$id_a[keep_a]
    names(id_a_dict) <- dictionary$gene_a[keep_a]
    id_b_dict <- dictionary$id_b[keep_b]
    names(id_b_dict) <- dictionary$gene_b[keep_b]
    id_dict <- c(id_a_dict, id_b_dict)
    id_dict <- id_dict[!duplicated(names(id_dict))]
    humanreadablename = c()
    for (i in row.names(sce_list2)) {
        humanreadablename = c(humanreadablename, as.character(unlist(id_dict[i])))
    }
    rownames(sce_list2) <- humanreadablename
    rownames(sce_list3) <- humanreadablename
    decon_subset <- deconvoluted[deconvoluted$complex_name %in% .findComplex(interactions_subset),
        ]
    if (nrow(decon_subset) > 0) {
        # although multiple rows are returned, really it's the same value for the same
        # complex
        decon_subset <- split(decon_subset, decon_subset$complex_name)
        decon_subset_expr <- lapply(decon_subset, function(x) {
            x <- x[, colnames(x) %in% colnames(sce_list2)]
            x <- colMeans(x)
            return(x)
        })
        if (!is.null(split.by)) {
            decon_subset_fraction <- lapply(decon_subset, function(x) {
                x <- unique(x$gene_name)
                test <- lapply(sce_list_alt, function(y) {
                  return(lapply(y, .cellTypeFraction_complex, genes = z, gene_symbol_mapping = gene_symbol_mapping))
                })
                return(test)
            })
            decon_subset_fraction <- lapply(decon_subset_fraction, function(x) {
                y <- lapply(x, function(z) do.call(cbind, z))
                for (i in 1:length(y)) {
                  colnames(y[[i]]) <- paste0(names(y[i]), "_", colnames(y[[i]]))
                }
                y <- do.call(cbind, y)
                return(y)
            })
        } else {
            decon_subset_fraction <- lapply(decon_subset, function(x) {
                z <- unique(x$gene_name)
                test <- lapply(sce_list_alt, function(y) {
                  return(.cellTypeFraction_complex(y, genes = z, gene_symbol_mapping = gene_symbol_mapping))
                })
                return(do.call(cbind, test))
            })
        }
        decon_subset_expr <- do.call(rbind, decon_subset_expr)
        decon_subset_fraction <- do.call(rbind, decon_subset_fraction)
        row.names(decon_subset_fraction) <- row.names(decon_subset_expr)
        requireNamespace("plyr")
        expr_df <- plyr::rbind.fill.matrix(sce_list2, decon_subset_expr)
        row.names(expr_df) <- c(row.names(sce_list2), row.names(decon_subset_expr))
        fraction_df <- plyr::rbind.fill.matrix(sce_list3, decon_subset_fraction)
        row.names(fraction_df) <- c(row.names(sce_list3), row.names(decon_subset_fraction))
    } else {
        expr_df <- sce_list2
        fraction_df <- sce_list3
    }
    # make a big fat edgelist
    if (!is.null(desiredInteractions)) {
        if (class(desiredInteractions) == "list") {
            desiredInteractions_ <- c(desiredInteractions, lapply(desiredInteractions,
                rev))
            cell_type_grid <- as.data.frame(do.call(rbind, desiredInteractions_))
        } else if ((class(desiredInteractions) == "data.frame")) {
            cell_type_grid <- desiredInteractions
        }
        cells_test = unique(unlist(desiredInteractions))
    } else {
        cells_test <- tryCatch(unique(droplevels(meta[, idents])), error = function(e) unique(meta[,
            idents]))
        cell_type_grid <- expand.grid(cells_test, cells_test)
    }
    if (remove_self) {
        rm_idx <- which(cell_type_grid[, 1] == cell_type_grid[, 2])
        if (length(rm_idx) > 0) {
            cell_type_grid <- cell_type_grid[-rm_idx, ]
        }
    }
    ligand <- interactions_subset$id_a
    receptor <- interactions_subset$id_b
    pair <- interactions_subset$interacting_pair
    converted_pair <- interactions_subset$converted
    receptor_a <- interactions_subset$receptor_a
    receptor_b <- interactions_subset$receptor_b
    producers <- as.character(cell_type_grid[, 1])
    receivers <- as.character(cell_type_grid[, 2])
    barcodes = paste0(lr_interactions$Var2, sep, lr_interactions$Var1)
    dfx <- list()
        if (!is.null(split.by)) {
        for (i in unique(meta[, split.by])) {
            dfx[[i]] <- .generateDf(ligand = ligand, sep = sep, receptor = receptor,
                receptor_a = receptor_a, receptor_b = receptor_b, pair = pair, converted_pair = converted_pair,
                producers = producers, receivers = receivers, cell_type_means = expr_df,
                cell_type_fractions = fraction_df, sce = sce_subset, sce_alt = sce_list_alt,
                gsm = gene_symbol_mapping, splitted = i)
            dfx[[i]] <- dfx[[i]][dfx[[i]]$barcode %in% barcodes, ]
        }
    } else {
        dfx[[1]] = .generateDf(ligand = ligand, sep = sep, receptor = receptor, receptor_a = receptor_a,
            receptor_b = receptor_b, pair = pair, converted_pair = converted_pair,
            producers = producers, receivers = receivers, cell_type_means = expr_df,
            cell_type_fractions = fraction_df, sce = sce_subset, sce_alt = sce_list_alt,
            gsm = gene_symbol_mapping)
        dfx[[1]] <- dfx[[1]][dfx[[1]]$barcode %in% barcodes, ]
    }
    if (return_df) {
        return(dfx)
    } else {
        # set the bundled connections
        df0 <- lapply(dfx, function(x) x[x$producer_fraction >= frac | x$receiver_fraction >= frac, ])  #save this for later
        # now construct the hierachy
        constructGraph <- function(input_group, sep, el, el0, unique_id, interactions_df,
            plot_cpdb_out, edge_group = FALSE, edge_group_colors = NULL, node_group_colors = NULL) {
            require(igraph)
            celltypes <- unique(c(as.character(el$producer), as.character(el$receiver)))
            el1 <- data.frame(from = "root", to = celltypes, barcode_1 = NA, barcode_2 = NA,
                barcode_3 = NA)
            el2 <- data.frame(from = celltypes, to = paste0(celltypes, sep, "ligand"),
                barcode_1 = NA, barcode_2 = NA, barcode_3 = NA)
            el3 <- data.frame(from = celltypes, to = paste0(celltypes, sep, "receptor"),
                barcode_1 = NA, barcode_2 = NA, barcode_3 = NA)
            el4 <- do.call(rbind, lapply(celltypes, function(x) {
                cell_ligands <- grep(x, el$from, value = TRUE)
                cell_ligands_idx <- grep(x, el$from)
                if (length(cell_ligands) > 0) {
                  df <- data.frame(from = paste0(x, sep, "ligand"), to = cell_ligands,
                    barcode_1 = el$barcode[cell_ligands_idx], barcode_2 = el$pair[cell_ligands_idx],
                    barcode_3 = paste0(el$from[cell_ligands_idx], sep, el$to[cell_ligands_idx]))
                } else {
                  df = NULL
                }
            }))
            el5 <- do.call(rbind, lapply(celltypes, function(x) {
                cell_ligands <- grep(x, el$to, value = TRUE)
                cell_ligands_idx <- grep(x, el$to)
                if (length(cell_ligands) > 0) {
                  df <- data.frame(from = paste0(x, sep, "receptor"), to = cell_ligands,
                    barcode_1 = el$barcode[cell_ligands_idx], barcode_2 = el$pair[cell_ligands_idx],
                    barcode_3 = paste0(el$from[cell_ligands_idx], sep, el$to[cell_ligands_idx]))
                } else {
                  df = NULL
                }
            }))
            gr_el <- do.call(rbind, list(el1, el2, el3, el4, el5))
            plot_cpdb_out$barcode <- paste0(plot_cpdb_out$Var2, sep, plot_cpdb_out$Var1)
            mean_col <- grep("means$", colnames(plot_cpdb_out), value = TRUE)
            means <- plot_cpdb_out[match(gr_el$barcode_1, plot_cpdb_out$barcode),
                mean_col]
            pval_col <- grep("pvals", colnames(plot_cpdb_out), value = TRUE)
            pvals <- plot_cpdb_out[match(gr_el$barcode_1, plot_cpdb_out$barcode),
                pval_col]
            gr_el <- cbind(gr_el, means, pvals)
            if (edge_group) {
                groups <- interactions_df$group[match(gr_el$barcode_2, interactions_df$interacting_pair)]
            }
            gr <- graph_from_edgelist(as.matrix(gr_el[, 1:2]))
            E(gr)$interaction_score <- as.numeric(means)
            E(gr)$pvals <- as.numeric(pvals)
            if (edge_group) {
                E(gr)$group <- groups
            }
            E(gr)$name <- gr_el$barcode_3
            # order the graph vertices
            V(gr)$type <- NA
            V(gr)$type[V(gr)$name %in% el4$to] <- "ligand"
            V(gr)$type[V(gr)$name %in% el5$to] <- "receptor"
            from = match(el0$from, V(gr)$name)
            to = match(el0$to, V(gr)$name)
            dat = data.frame(from = el0$from, to = el0$to)
            if (nrow(dat) > 0) {
                dat$barcode = paste0(dat$from, sep, dat$to)
                interaction_score = E(gr)$interaction_score[match(dat$barcode, gr_el$barcode_3)]
                pval = E(gr)$pvals[match(dat$barcode, gr_el$barcode_3)]
                if (any(is.na(pval))) {
                  pval[is.na(pval)] <- 1
                }
                if (!all(is.na(range01(-log10(pval))))) {
                  pval <- range01(-log10(pval))
                }
                if (edge_group) {
                  group = E(gr)$group[match(dat$barcode, gr_el$barcode_3)]
                }
                ligand_expr <- data.frame(cell_mol = el$from, expression = el$producer_expression,
                  fraction = el$producer_fraction)
                recep_expr <- data.frame(cell_mol = el$to, expression = el$receiver_expression,
                  fraction = el$receiver_fraction)
                expression <- rbind(ligand_expr, recep_expr)
                df <- igraph::as_data_frame(gr, "both")
                df$vertices$expression <- 0
                df$vertices$fraction <- 0
                df$vertices$expression <- as.numeric(expression$expression)[match(df$vertices$name,
                  expression$cell_mol)]
                df$vertices$fraction <- as.numeric(expression$fraction)[match(df$vertices$name,
                  expression$cell_mol)]
                df$vertices$celltype <- ""
                for (x in cells_test) {
                  idx <- grepl(paste0(x, sep), df$vertices$name)
                  df$vertices$celltype[idx] <- x
                }
                df$vertices$label <- df$vertices$name
                df$vertices$label[!df$vertices$name %in% c(el0$from, el0$to)] <- ""
                gr <- graph_from_data_frame(df$edges, directed = TRUE, vertices = df$vertices)
                for (x in unique_id) {
                  V(gr)$label <- gsub(paste0(x, sep), "", V(gr)$label)
                }
                require(ggraph)
                require(ggrepel)
                if (!is.null(edge_group_colors)) {
                  edge_group_colors = edge_group_colors
                } else {
                  nn = length(unique(E(gr)$group))
                  edge_group_colors = .gg_color_hue(nn)
                }
                if (!is.null(node_group_colors)) {
                  node_group_colors = node_group_colors
                } else {
                  nn = length(unique(meta[, idents]))
                  node_group_colors = .gg_color_hue(nn)
                }
                # plot the graph
                if (edge_group) {
                  pl <- ggraph(gr, layout = "dendrogram", circular = TRUE) + geom_conn_bundle(data = get_con(from = from,
                    to = to, group = group, `-log10(sig)` = pval, interaction_score = interaction_score),
                    aes(colour = group, alpha = interaction_score, width = `-log10(sig)`),
                    tension = 0.5) + # scale_edge_width(range = c(1, 3)) + scale_edge_alpha(limits = c(0, 1)) +
                  scale_edge_color_manual(values = edge_group_colors) + geom_node_point(pch = 19,
                    aes(size = fraction, filter = leaf, color = celltype, alpha = type)) +
                    theme_void() + coord_fixed() + scale_size_continuous(limits = c(0,
                    1)) + scale_shape_manual(values = c(ligand = 19, receptor = 15)) +
                    scale_color_manual(values = node_group_colors) + geom_text_repel(aes(x = x,
                    y = y, label = label), segment.square = TRUE, segment.inflect = TRUE,
                    segment.size = 0.2, force = 0.5, size = 2, force_pull = 0) +
                  scale_alpha_manual(values = c(ligand = 0.5, receptor = 1)) + small_legend(keysize = 0.5) +
                    ggtitle(input_group)
                } else {
                  pl <- ggraph(gr, layout = "dendrogram", circular = TRUE) + geom_conn_bundle(data = get_con(from = from,
                    to = to, `-log10(sig)` = pval, interaction_score = interaction_score),
                    aes(alpha = interaction_score, width = `-log10(sig)`), tension = 0.5) +
                    # scale_edge_width(range = c(1, 3)) + scale_edge_alpha(limits = c(0, 1)) +
                  scale_edge_color_manual(values = edge_group_colors) + geom_node_point(pch = 19,
                    aes(size = fraction, filter = leaf, color = celltype, alpha = type)) +
                    theme_void() + coord_fixed() + scale_size_continuous(limits = c(0,
                    1)) + scale_shape_manual(values = c(ligand = 19, receptor = 15)) +
                    scale_color_manual(values = node_group_colors) + geom_text_repel(aes(x = x,
                    y = y, label = label), segment.square = TRUE, segment.inflect = TRUE,
                    segment.size = 0.2, force = 0.5, size = 2, force_pull = 0) +
                    # geom_node_text(aes(x = x*1.15, y=y*1.15, filter = leaf, label=label, size
                  # =0.01)) +
                  scale_alpha_manual(values = c(ligand = 0.5, receptor = 1)) + small_legend(keysize = 0.5) +
                    ggtitle(input_group)
                }
                return(pl)
            } else {
                return(NA)
            }
        }
        gl <- list()
        if (!is.null(interaction_grouping)) {
            edge_group = TRUE
        } else {
            edge_group = FALSE
        }
        cantplot <- c()
        noplot <- FALSE
        for (i in 1:length(dfx)) {
            if (!is.null(split.by)) {
                if (nrow(dfx[[i]]) > 0 & nrow(df0[[i]]) > 0) {
                  gl[[i]] <- constructGraph(names(dfx)[i], sep, dfx[[i]], df0[[i]],
                    cells_test, interactions_subset, lr_interactions, edge_group,
                    edge_group_colors, node_group_colors)
                } else {
                  gl[[i]] <- NA
                  cantplot <- c(cantplot, names(dfx)[i])
                }
            } else {
                if (nrow(dfx[[i]]) > 0 & nrow(df0[[i]]) > 0) {
                  gl[[i]] <- constructGraph(NULL, sep, dfx[[i]], df0[[i]], cells_test,
                    interactions_subset, lr_interactions, edge_group, edge_group_colors,
                    node_group_colors)
                } else {
                  gl[[i]] <- NA
                  noplot <- TRUE
                }
            }
        }
        if (length(cantplot) > 0) {
            cat("The following groups in split.by cannot be plotted due to missing/no significant interactions/celltypes",
                sep = "\n")
            cat(cantplot, sep = "\n")
        }
        if (noplot) {
            cat("Please check that plot_cpdb works first!", sep = "\n")
        }
        if (length(gl) > 1) {
            return(gl)
        } else {
            return(gl[[1]])
        }
    }
}